import { AbstractProperty } from "../../properties/abstract-property";
import { PropertyDependencyOptions } from "../../dependency-graph/property-dependency";
import { AttributeId } from "../../attributes/attribute-id";
import { Rule } from "../../rules/rule";
import { TextInterpreter, TextInterpreterFcn } from "../../util/text-interpreter/text-interpreter";
import { AbstractPropertyImpl } from "../../properties/abstract-property-impl";
import { PropertyValidator } from "../../validators/property-validator";
import { AsyncPropertyValidator } from "../../validators/async-property-validator";
export declare abstract class AbstractPropertyRuleBuilder<D, Property extends AbstractPropertyImpl<D>> {
    protected readonly property: Property;
    protected readonly addDependencies: (from: readonly AbstractProperty[], to: AbstractProperty, options: PropertyDependencyOptions) => void;
    protected readonly textInterpreters: {
        [textInterpreter in TextInterpreter]?: TextInterpreterFcn;
    };
    constructor(property: Property, addDependencies: (from: readonly AbstractProperty[], to: AbstractProperty, options: PropertyDependencyOptions) => void, textInterpreters: {
        [textInterpreter in TextInterpreter]?: TextInterpreterFcn;
    });
    protected addValidatorInternal<Dependencies extends readonly AbstractProperty[]>(...dependencies: Dependencies): (validator: PropertyValidator<Property, Dependencies>) => this;
    protected addAsyncValidatorInternal<Dependencies extends readonly AbstractProperty[]>(...dependencies: Dependencies): (validator: AsyncPropertyValidator<Property, Dependencies>) => this;
    set<A>(attribudeId: AttributeId<A>, value: A): this;
    define<A, Dependencies extends readonly AbstractProperty[]>(attribudeId: AttributeId<A>, ...dependencies: Dependencies): (value: Rule<[self: Property, ...dependencies: Dependencies], A>) => this;
    private defineAttributeFunction;
    setVisibility(isVisible: boolean): this;
    defineVisibility<Dependencies extends readonly AbstractProperty[]>(...dependencies: Dependencies): (isVisible: Rule<[self: Property, ...dependencies: Dependencies], boolean>) => this;
    private defineVisibilityFunction;
    defineLabel(text: string, textInterpreter?: TextInterpreter): this;
    protected interpreteText(text: string, textInterpreter?: TextInterpreter): string;
    protected checkTextInterpreterAvailable(textInterpreter?: TextInterpreter): void;
    onUpdated(updated: (self: Property) => void): this;
    setToInitialStateOnOtherPropertyChanged(...otherProperties: AbstractProperty[]): this;
}
