"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PropertyScalarRuleBuilder = void 0;
const assertions_1 = require("../../util/assertions/assertions");
const attribute_id_1 = require("../../attributes/attribute-id");
const abstract_property_rule_builder_impl_1 = require("./abstract-property-rule-builder-impl");
class PropertyScalarRuleBuilder extends abstract_property_rule_builder_impl_1.AbstractPropertyRuleBuilder {
    constructor(property, notEmptyIfRequiredValidator, addDependencies, textInterpreters) {
        super(property, addDependencies, textInterpreters);
        this.notEmptyIfRequiredValidator = notEmptyIfRequiredValidator;
    }
    addValidator(...dependencies) {
        return this.addValidatorInternal(...dependencies);
    }
    addAsyncValidator(...dependencies) {
        return this.addAsyncValidatorInternal(...dependencies);
    }
    defineInitialValue(value) {
        (0, assertions_1.alwaysAssertThat)(!this.property.isReadOnly(), () => `${this.property.id}: Can not define initial value on read only property`);
        this.property.defineInitialValue(value);
        this.property.setToInitialState();
        return this;
    }
    setRequiredIfVisible(required) {
        if (required) {
            this.defineRequiredIfVisibleFunction([], () => required);
        }
        else {
            this.property.defineRequiredIfVisible(undefined);
        }
        return this;
    }
    defineRequiredIfVisible(...dependencies) {
        return (required) => {
            this.defineRequiredIfVisibleFunction(dependencies, () => required(this.property, ...dependencies));
            return this;
        };
    }
    defineRequiredIfVisibleFunction(dependencies, fcn) {
        this.addDependencies(dependencies, this.property, { required: true });
        this.property.defineRequiredIfVisible({
            id: attribute_id_1.A.Required,
            dependencies: dependencies,
            getValue: fcn
        });
        this.property.addPropertyValidator(this.notEmptyIfRequiredValidator, []);
    }
    defineLabelAndPlaceholder(text) {
        this.defineLabel(text);
        return this.definePlaceholder(text);
    }
    definePlaceholder(placeholder) {
        this.property.definePlaceholder(placeholder);
        return this;
    }
    defineInfoText(text, textInterpreter) {
        this.checkTextInterpreterAvailable(textInterpreter);
        this.property.defineInfoText(this.interpreteText(text, textInterpreter));
        return this;
    }
}
exports.PropertyScalarRuleBuilder = PropertyScalarRuleBuilder;
