import { AbstractProperty } from "../properties/abstract-property";
import { RuleEngineUpdateHandler } from "./rule-engine-update-handler-impl";
import { Snapshot } from "./snapshot/snapshot";
import { BuilderOptions } from "./builder/builder-options";
import { Builder } from "./builder/builder";
import { CrossValidationResult } from "../validators/cross-validation-result";
import { CrossValidatorInstance } from "./validation/validator-instance-impl";
import { AbstractDataProperty } from "../properties/abstract-data-property";
import { PropertyGroup } from "../properties/group-of-properties";
import { RuleSet } from "./rule-set/rule-set";
import { RuleEngine } from "./rule-engine";
import { ValidationMessagesMap } from "../validators/validation-messages-map";
import { ValidationResult } from "../validators/validation-result";
import { PropertyId } from "../properties/property-id";
import { RuleEngineData } from "./data/rule-engine-data";
import { RulesVersion } from "./data/rules-version";
export declare class RuleEngineImpl implements RuleEngine, RuleEngineUpdateHandler {
    private readonly builder;
    private readonly propertyMap;
    private readonly dependencyGraph;
    private readonly version;
    private readonly dataMigrators?;
    private readonly validations;
    private readonly dataLinks;
    private readonly snapshots;
    private get properties();
    constructor(options: BuilderOptions);
    getVersion(): RulesVersion;
    private isCompatible;
    getBuilder(): Builder;
    getPropertyById(id: PropertyId): AbstractDataProperty<unknown> | undefined;
    validateAllProperties(): Promise<ValidationResult>;
    setValidationMessages(validationMessages: ValidationMessagesMap | ValidationResult): PropertyId[];
    getValidationMessages(): ValidationResult;
    clearValidationResult(): void;
    exportData(onlySelectedProperties?: AbstractDataProperty<unknown>[]): RuleEngineData;
    importData(data: RuleEngineData): void;
    setToInitialState(): void;
    takeSnapShot(key?: string): Snapshot;
    restoreSnapShot(key?: string): void;
    linkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    unlinkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    private createDataLinkKey;
    needsAnUpdate(mightHaveChanged: AbstractProperty): void;
    updateValue(property: AbstractProperty): Promise<void> | undefined;
    private updateValueInternal;
    private hasBeenUpdated;
    cancelValidationAndInvalidateResults(validators: readonly CrossValidatorInstance<readonly AbstractProperty[]>[]): void;
    validateValidatorInstances(validators: readonly CrossValidatorInstance<readonly AbstractProperty[]>[]): Promise<CrossValidationResult | 'cancelled'>[];
    private getValidationProcess;
    removeOwnedProperties(ownerId: PropertyId, ownedProperties?: PropertyId[]): void;
    defineRuleSet<S extends PropertyGroup>(initFcn: (builder: Builder) => S): RuleSet<S>;
}
