import { AbstractProperty } from "./abstract-property";
import { ValueChangeListener, ValueChangeListenerReference } from "./value-change-listener";
import { Trigger } from "./trigger";
import { RuleEngineUpdateHandler } from "../engine/rule-engine-update-handler-impl";
import { ValidationMessage } from "../validators/validation-message";
import { PropertyDependency } from "../dependency-graph/property-dependency";
import { BackpressureConfig } from "./backpressure/backpressure-config";
import { CrossValidatorInstance } from "../engine/validation/validator-instance-impl";
import { AbstractDataProperty } from "./abstract-data-property";
import { PropertyValidator } from "../validators/property-validator";
import { AsyncPropertyValidator } from "../validators/async-property-validator";
import { AttributeId } from "../attributes/attribute-id";
import { Attribute } from "../attributes/attribute";
import { ValidationResult } from "../validators/validation-result";
import { ValidationMessagesMap } from "../validators/validation-messages-map";
export interface AbstractPropertyWithInternals<D> extends AbstractDataProperty<D> {
    internallyUpdate(): Promise<void> | void;
    hasBeenUpdated(): void;
    errorWhileUpdating(error: any): void;
    dependencyHasBeenUpdated(dependency: PropertyDependency): void;
    addValidator<Properties extends readonly AbstractProperty[]>(validator: CrossValidatorInstance<Properties>): void;
}
export declare abstract class AbstractPropertyImpl<D> implements AbstractPropertyWithInternals<D> {
    protected updateHandler: RuleEngineUpdateHandler;
    backpressureConfig?: BackpressureConfig | undefined;
    private automaticallyUpdate;
    private manuallyTriggered?;
    private triggerListener?;
    private touched?;
    private needsToRecompute?;
    private recomputingCount?;
    private currentRecomputing?;
    private updatedListeners?;
    private crossValidators?;
    private validationMessages;
    private valueChangeListeners?;
    private nextValueChangeListenerId?;
    private propertyValidators?;
    private asyncPropertyValidators?;
    abstract id: string;
    private label?;
    private attributeMap?;
    private visible?;
    constructor(updateHandler: RuleEngineUpdateHandler, backpressureConfig?: BackpressureConfig | undefined);
    protected abstract internallySyncUpdate(): void;
    protected abstract internallyAsyncUpdate<V>(): {
        asyncPromise: Promise<V>;
        resolve: (value: V) => void;
    };
    protected awaitAsyncUpdate(): Promise<void> | undefined;
    private updatingPromise;
    private nextRecomputingCount;
    protected syncUpdateIfNeeded(): void;
    private getTriggerListener;
    internallyUpdate(): Promise<void> | void;
    private handleBackpressure;
    private handleAsyncUpdateWithDebounceTime;
    private handleAsyncUpdate;
    needsAnUpdate(notifyOthers?: boolean, needsToRecompute?: boolean): void;
    hasBeenUpdated(): void;
    errorWhileUpdating(error: any): void;
    dependencyHasBeenUpdated(dependency: PropertyDependency): void;
    addPropertyValidator<Dependencies extends readonly AbstractProperty[]>(validator: PropertyValidator<any, any>, dependencies: Dependencies): void;
    addAsyncPropertyValidator<Dependencies extends readonly AbstractProperty[]>(validator: AsyncPropertyValidator<any, any>, dependencies: Dependencies): void;
    protected getPropertyValidatorResults(): ValidationMessage[];
    protected getAsyncPropertyValidatorResults(): Promise<ValidationMessage[]>;
    addValidator<Properties extends readonly AbstractProperty[]>(validator: CrossValidatorInstance<Properties>): void;
    validate(): Promise<ValidationMessage[]>;
    validateRecursively(): Promise<ValidationResult>;
    validateRecursivelyInternal(): Promise<ValidationMessagesMap>;
    isValid(): boolean;
    getValidationMessages(): ValidationMessage[];
    setValidationMessages(messages: ValidationMessage[]): void;
    clearValidationResult(): void;
    private cancelValidationAndInvalidateResults;
    private updateValidationMessages;
    registerValueChangedListener(changed: ValueChangeListener): ValueChangeListenerReference;
    deregisterValueChangedListener(changed?: ValueChangeListenerReference): void;
    protected hasValueChangeListeners(): boolean;
    protected tellValueChangeListeners(fcn: (listener: ValueChangeListener) => void): void;
    setUpdateStrategy(strategy: 'Automatic' | 'Manually'): void;
    addUpdateTrigger(trigger: Trigger, setUpdateStrategyManually?: boolean): void;
    removeUpdateTrigger(trigger: Trigger): void;
    setTouched(touched: boolean): void;
    isTouched(): boolean;
    defineLabel(label: string): void;
    getLabel(): string;
    defineAttribute(attribute: Attribute<unknown>): void;
    get<A>(id: AttributeId<A>): A | undefined;
    defineVisibility(visibility?: Attribute<boolean>): void;
    isVisible(): boolean;
    transferData(toProperty: AbstractDataProperty<D>): void;
    setToInitialState(): void;
    protected abstract setDataToInitialState(): void;
    protected removeOwnedProperties(properties?: AbstractProperty[]): void;
    abstract isAsynchronous(): boolean;
    abstract isProcessing(): boolean;
    abstract isReadOnly(): boolean;
    abstract exportData(): D | null;
    abstract importData(data: D | null): void;
}
