import { PropertyId } from "./property-id";
import { ValueChangeListener, ValueChangeListenerReference } from "./value-change-listener";
import { Trigger } from "./trigger";
import { BackpressureConfig } from "./backpressure/backpressure-config";
import { ValidationMessage } from "../validators/validation-message";
import { AttributeId } from "../attributes/attribute-id";
import { ValidationResult } from "../validators/validation-result";
export interface AbstractProperty {
    readonly id: PropertyId;
    needsAnUpdate(notifyOthers?: boolean): void;
    registerValueChangedListener(changed: ValueChangeListener): ValueChangeListenerReference;
    deregisterValueChangedListener(changed?: ValueChangeListenerReference): void;
    setUpdateStrategy(strategy: 'Automatic' | 'Manually'): void;
    addUpdateTrigger(trigger: Trigger): void;
    addUpdateTrigger(trigger: Trigger, setUpdateStrategyManually: boolean): void;
    removeUpdateTrigger(trigger: Trigger): void;
    setTouched(touched: boolean): void;
    isTouched(): boolean;
    getLabel(): string;
    get<A>(id: AttributeId<A>): A | undefined;
    isVisible(): boolean;
    validate(): Promise<ValidationMessage[]>;
    validateRecursively(): Promise<ValidationResult>;
    isValid(): boolean;
    setValidationMessages(messages: ValidationMessage[]): void;
    getValidationMessages(): ValidationMessage[];
    clearValidationResult(): void;
    isAsynchronous(): boolean;
    backpressureConfig?: BackpressureConfig;
    isProcessing(): boolean;
    isReadOnly(): boolean;
}
