"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractRulesDefinition = exports.RulesDefinitionComposition = exports.RulesDefinitionWithDependencies = void 0;
class RulesDefinitionWithDependencies {
    constructor(dependencies, buildRulesFcn) {
        this.dependencies = dependencies;
        this.buildRulesFcn = buildRulesFcn;
    }
    buildRules(builder) {
        this.buildRulesFcn(builder, ...this.dependencies);
    }
}
exports.RulesDefinitionWithDependencies = RulesDefinitionWithDependencies;
class RulesDefinitionComposition {
    constructor(definitions) {
        this.definitions = definitions;
    }
    buildRules(builder) {
        this.definitions.forEach(def => def.buildRules(builder));
    }
}
exports.RulesDefinitionComposition = RulesDefinitionComposition;
class AbstractRulesDefinition {
    buildRules(builder) {
        if (!this.implementation) {
            throw new Error("No implementation provided");
        }
        this.implementation.buildRules(builder);
    }
    implementWith(implementation) {
        this.implementation = implementation;
    }
}
exports.AbstractRulesDefinition = AbstractRulesDefinition;
