import { alwaysAssertThat } from "../../util/assertions/assertions";
import { A } from "../../attributes/attribute-id";
import { AbstractPropertyRuleBuilder } from "./abstract-property-rule-builder-impl";
export class PropertyScalarRuleBuilder extends AbstractPropertyRuleBuilder {
    constructor(property, notEmptyIfRequiredValidator, addDependencies, textInterpreters) {
        super(property, addDependencies, textInterpreters);
        this.notEmptyIfRequiredValidator = notEmptyIfRequiredValidator;
    }
    addValidator(...dependencies) {
        return this.addValidatorInternal(...dependencies);
    }
    addAsyncValidator(...dependencies) {
        return this.addAsyncValidatorInternal(...dependencies);
    }
    defineInitialValue(value) {
        alwaysAssertThat(!this.property.isReadOnly(), () => `${this.property.id}: Can not define initial value on read only property`);
        this.property.defineInitialValue(value);
        this.property.setToInitialState();
        return this;
    }
    setRequiredIfVisible(required) {
        if (required) {
            this.defineRequiredIfVisibleFunction([], () => required);
        }
        else {
            this.property.defineRequiredIfVisible(undefined);
        }
        return this;
    }
    defineRequiredIfVisible(...dependencies) {
        return (required) => {
            this.defineRequiredIfVisibleFunction(dependencies, () => required(this.property, ...dependencies));
            return this;
        };
    }
    defineRequiredIfVisibleFunction(dependencies, fcn) {
        this.addDependencies(dependencies, this.property, { required: true });
        this.property.defineRequiredIfVisible({
            id: A.Required,
            dependencies: dependencies,
            getValue: fcn
        });
        this.property.addPropertyValidator(this.notEmptyIfRequiredValidator, []);
    }
    defineLabelAndPlaceholder(text) {
        this.defineLabel(text);
        return this.definePlaceholder(text);
    }
    definePlaceholder(placeholder) {
        this.property.definePlaceholder(placeholder);
        return this;
    }
    defineInfoText(text, textInterpreter) {
        this.checkTextInterpreterAvailable(textInterpreter);
        this.property.defineInfoText(this.interpreteText(text, textInterpreter));
        return this;
    }
}
