import { AbstractProperty } from "../properties/abstract-property";
import { Snapshot } from "./snapshot/snapshot";
import { Builder } from "./builder/builder";
import { PropertyGroup } from "../properties/group-of-properties";
import { RuleSet } from "./rule-set/rule-set";
import { BuilderOptions } from "./builder/builder-options";
import { AbstractDataProperty } from "../properties/abstract-data-property";
import { RuleEngineImpl } from "./rule-engine-impl";
import { PropertyId } from "../properties/property-id";
import { ValidationMessagesMap } from "../validators/validation-messages-map";
import { ValidationResult } from "../validators/validation-result";
import { RuleEngineData } from "./data/rule-engine-data";
import { RulesVersion } from "./data/rules-version";
export declare function createRuleEngine(options: BuilderOptions): RuleEngineImpl;
export interface RuleEngine {
    getVersion(): RulesVersion;
    getBuilder(): Builder;
    getPropertyById(id: PropertyId): AbstractDataProperty<unknown> | undefined;
    validateAllProperties(): Promise<ValidationResult>;
    setValidationMessages(validationMessages: ValidationMessagesMap | ValidationResult): PropertyId[];
    getValidationMessages(): ValidationResult;
    clearValidationResult(): void;
    exportData(onlySelectedProperties?: AbstractDataProperty<unknown>[]): RuleEngineData;
    importData(data: RuleEngineData): void;
    setToInitialState(): void;
    takeSnapShot(key?: string): Snapshot;
    restoreSnapShot(key?: string): void;
    linkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    unlinkPropertyData<D>(propertyA: AbstractDataProperty<D>, propertyB: AbstractDataProperty<D>): void;
    needsAnUpdate(mightHaveChanged: AbstractProperty): void;
    defineRuleSet<S extends PropertyGroup>(initFcn: (builder: Builder) => S): RuleSet<S>;
}
