export class ValidationResult {
    constructor(validationMessagesMap, getPropertyById) {
        this.validationMessagesMap = validationMessagesMap;
        this.getPropertyById = getPropertyById;
    }
    getValidationMessagesMap() {
        return this.validationMessagesMap;
    }
    getAllMessages() {
        return Object.values(this.getValidationMessagesMap()).flat();
    }
    getAllMessagesAsStrings(convert) {
        return Object.entries(this.getValidationMessagesMap()).flatMap(([id, messages]) => {
            const property = this.getPropertyById(id);
            return messages.map(message => convert(message, property));
        });
    }
    isValid() {
        if (this.valid === undefined) {
            this.valid = Object.values(this.getValidationMessagesMap()).every(messages => messages.every(message => message.type.isValid));
        }
        return this.valid;
    }
    filterByType(...types) {
        return this.filterBy(msg => !!types.find(type => msg.type.name === type.name));
    }
    filterValid() {
        return this.filterBy(msg => msg.type.isValid);
    }
    filterInvalid() {
        return this.filterBy(msg => !msg.type.isValid);
    }
    filterBy(fcn) {
        const validationMap = Object.keys(this.validationMessagesMap).reduce((res, key) => {
            const filteredMsgs = this.validationMessagesMap[key].filter(msg => fcn(msg));
            if (filteredMsgs.length > 0) {
                res[key] = filteredMsgs;
            }
            return res;
        }, {});
        return new ValidationResult(validationMap, this.getPropertyById);
    }
}
